!/ Origination: Apr 2015
!/ Author     : Brandon G. Reichl
!/
      MODULE KPP
      IMPLICIT NONE

      private
      include 'pom.h'
      ! Similarity coefficients
      real, parameter:: zetas = -1.0
      real, parameter:: zetam = -0.2
      real, parameter:: cs    = 98.96
      real, parameter:: Cm    = 8.38
      real, parameter:: As    = -28.86
      real, parameter:: Am    = 1.26
      ! For non-local flux (Cs differs from cs, naming 'See-es')
      ! Solved for from Cs= kappa * cstar * (kappa * epsilon * cs)^(1/3)
      real, parameter:: SEEes = 3.1637!6.3254
      ! Heat capacity of water [J/kg/degC]
      real, parameter:: Cp    = 3985 
      ! Critical gradient Richardson number for turbulent mixing
      real, parameter :: Ri0 = 0.7
      ! Max interior viscosity/diffusivity due to shear instability
      real, parameter :: KMRIg = 10.0E-4
      real, parameter :: KHRIg = 10.0E-4
      ! Interior viscosity due to internal wave breaking
      real, parameter :: KMWB = 1.e-5
      real, parameter :: KHWB = 1.e-6
      !  buoyancy frequency (1/s2) limit for convection
      real, parameter :: bvfcon  = -2.e-5
      ! Interior viscosity due to convective diffusivity due to shear instability
      real, parameter :: KC = 0.01
      ! Switches
        LOGICAL :: SW_HFLUX    = .true.
        LOGICAL :: SW_Matching = .true.
        LOGICAL :: DIAGNOSTIC  = .false.
        LOGICAL :: KPP_SBL     = .true.
        LOGICAL :: KPP_BBL     = .true.
        LOGICAL :: USE_KPP_K   = .true.
        integer, parameter :: KPPLAGRANGIAN=0
!
      public:: ProfKPP
      contains

      Subroutine profKPP
        USE LINEAR_MD
        !Interface to KPP
        integer :: indi, indj, indk
        integer,dimension(im,jm)  :: KPPhind
        real, dimension(im,jm)    :: ustar2,hek,hlim
        real, dimension(im,jm,kb) :: KPPRIB
        real, dimension(im,jm) :: BSTAR, Bflux, HFLUX, SFLUX
        REAL :: ALPHA_TE        !Thermal Expansion Coefficient [1/degC]
        REAL :: STKM
        real dpt(1), botu(1),botv(1),DU,DV,SHEAR_DIR,WAVE_DIR,
     $        SHEAR_WAVE_ANG
        real r(5),ad1(5),ad2(5)
        REAL TOUTTEST
! irradiance parameters after Paulson and Simpson (1977)
!       ntp               1      2       3       4       5
!   Jerlov type           i      ia      ib      ii     iii
      data r   /       .58e0,  .62e0,  .67e0,  .77e0,  .78e0 /
      data ad1 /       .35e0,  .60e0,  1.0e0,  1.5e0,  1.4e0 /
      data ad2 /       23.e0,  20.e0,  17.e0,  14.e0,  7.9e0 /

      KPPHIND=1
      KMKPP=0.0
      KHKPP=0.0
      KPPRIB = 0.0
      BSTAR = 0.0
      BFLUX = 0.0
      HFLUX = 0.0
      SFLUX = 0.0
      hek =0.0
      ustar2 = 0.0
      
!/---------------------------------------------------------------------+   
! Get u*, B*, hek, and hmon
!\---------------------------------------------------------------------+
        if (ionedim.eq.0) then
           do indj=1,jmm1
              do indi=1,imm1
                 ustar2(indi,indj) = sqrt((.5e0*(wusurf(indi,indj)
     $                * dum(indi,indj) + wusurf(indi+1,indj)
     $                * dum(indi+1,indj)))**2 + 
     $                (.5e0*(wvsurf(indi,indj)*dvm(indi,indj)
     $                +wvsurf(indi,indj+1)*dvm(indi,indj+1)))**2)
              enddo
           enddo
        else
           do indj=1,jmm1
              do indi=1,imm1
                 ustar2(indi,indj) = sqrt((wusurf(indi,indj)
     $                * dum(indi,indj))**2 +
     $                (wvsurf(indi,indj)*dvm(indi,indj))**2)
              enddo
           enddo
        endif

        do indj=1,jmm1
           do indi=1,imm1
              hek(indi,indj) = 0.7 * sqrt(ustar2(indi,indj))
     $             / max(SMALL,abs(cor(indi,indj)))
              ! Get Total surface heat flux in W/m^2
              if (SW_HFLUX) then
                 !HFLUX(indi,indj) = -50.0 !(positive upwards)
                 HFLUX(indi,indj)=(wtsurf(indi,indj)-swrad(indi,indj))
     $                *rhoref*cp
              else
                 HFLUX(indi,indj) = 0.0
              endif
              !Get Alpha_te (thermal expansion coefficient) in 1/degC
              CALL GETALPHA(S(indi,indj,1),T(indi,indj,1),0.0,alpha_te)
              !Get Short-wave radiation flux remaining at base of surface layer
              ! this is subtracted from surface flux due to POM convention.
              BR(indi,indj)=swrad(indi,indj)
     $             *(1.-( r(ntp)     *exp(-kpph(indi,indj)/ad1(ntp) )
     $             + (1.e0-r(ntp))*exp(-kpph(indi,indj)/ad2(ntp))))
     $             *rhoref*Cp
              ! Get Bf in m^2/s^3
              Bflux(indi,indj) = (HFLUX(indi,indj)+BR(indi,indj))*
     $             alpha_te * grav/rhoref/Cp;
!              ! Get Bstar in m/s^2
!              Bstar(indi,indj) = Bflux(indi,indj)
!     $             /max(small,sqrt(ustar2(indi,indj)))
              ! Get Minon-Obhukov length in m
              if (abs(hflux(indi,indj)).ge.0.01) then
                 hmo(indi,indj) = 1.0 * sqrt(ustar2(indi,indj))*
     $                ustar2(indi,indj) / kappa / BFLUX(indi,indj)
              else
                 hmo(indi,indj) =sign(10000.0,HFLUX(indi,indj))
              endif
           enddo
        enddo      
        !print*,'BR',minval(BR),maxval(BR)
        !print*,'Bflux',minval(BFLUX),maxval(Bflux)
        call exchange2d_mpi(ustar2,im,jm)
        call exchange2d_mpi(hek,im,jm)
        call exchange2d_mpi(hmo,im,jm)
        call exchange2d_mpi(hflux,im,jm)
        call exchange2d_mpi(bflux,im,jm)
        call exchange2d_mpi(BR,im,jm)
        hlim=hek
!/---------------------------------------+
! Calculate Shear and buoyancy frequency
!\---------------------------------------+

        if (diagnostic) then
           !print*,'Compute NN/SS'
        endif
        call Freq_Prof(SS,NN)
        if (diagnostic) then
           print*,'NN/SS',maxval(SS),maxval(NN)
        endif
        do indk=1,kb
           call exchange2d_mpi(SS(:,:,indk),im,jm)
           call exchange2d_mpi(NN(:,:,indk),im,jm)
        enddo

!/-----------------------------+
!Compute interior diffusivities
!\-----------------------------+
        if (diagnostic) then
           !print*,'Interior'
        endif
        call interiorK(SS,NN)

!/---------------------------------------------------------------------+
! Construct Stokes current onto POM vertical grid
!\---------------------------------------------------------------------+
        LA(:,:)=100000.
        do indj=1,jmm1
           do indi=1,imm1
              US(indi,indj,:)=0.0
              VS(indi,indj,:)=0.0
              if (KPPLT.eq.1) then
                 STKM=sqrt(LASTX(indi,indj)**2+LASTY(indi,indj)**2)
                 if (STKM.gt.0.001) then
                    DPT(1) = KPPh(indi,indj)*.2/h(indi,indj)
                   call linear(-999.,-999.,u(indi,indj,:),
     $                  BOTU(1),-zz,DPT(1),kb,1)
                   call linear(-999.,-999.,v(indi,indj,:),
     $                  BOTV(1),-zz,DPT(1),kb,1)
                   DU = U(indi,indj,1)-BOTU(1)
                   DV = V(indi,indj,1)-BOTV(1)
                   call linear(-999.,-999.,u(indi+1,indj,:),
     $                  BOTU(1),-zz,DPT(1),kb,1)
                   call linear(-999.,-999.,v(indi,indj+1,:),
     $                  BOTV(1),-zz,DPT(1),kb,1)
                   DU =DU+U(indi+1,indj,1)-BOTU(1)
                   DV =DV+V(indi,indj+1,1)-BOTV(1)
                   DU = DU * 0.5
                   DV = DV * 0.5
                   SHEAR_DIR = atan2(DV,DU)
                   WAVE_DIR  = atan2(.5*(LASTY(indi,indj)
     $                  +LASTY(indi,indj+1)),
     $                  .5*(LASTX(indi,indj)+LASTX(indi+1,indj)))
                   SHEAR_WAVE_ANG = max(0.0001,cos(SHEAR_DIR-WAVE_DIR))
                   LA(indi,indj) = max(0.1,
     $                  sqrt(max(0.001,sqrt(ustar2(indi,indj)))/STKM))
                    LA(indi,indj) = LA(indi,indj)*min(10.0,
     $                   (sqrt(1./SHEAR_WAVE_ANG)))
                 else
                    LA(indi,indj) = 100000.0
                 endif
              else
                 LA(indi,indj)=100000.0
              endif
           enddo
        enddo
        !print*,'STK',maxval(LASTX),maxval(LASTY)
        !print*,'LA',minval(LA),maxval(LA)
        call exchange2d_mpi(LA,im,jm)
!/---------------------------------------------------------------------+
! Get RIb
!\---------------------------------------------------------------------+
        if (KPP_SBL) then
           if (diagnostic) then
              !print*,'Get bulk Richardson number'
           endif
           call ComputeRIb(NN,USTAR2,BFLUX,KPPRIB)
           if (diagnostic) then
              print*,'Rib',maxval(KPPRIB),maxval(KPPRIB)
           endif
        else
           KPPRIB=0.0
        endif
        do indk=1,kb
           call exchange2d_mpi(KPPRIB(:,:,indk),im,jm)
        enddo
!/---------------------------------------------------------------------+
! Get KPP surface mixing depth
!\---------------------------------------------------------------------+
        if (KPP_SBL) then
           if (diagnostic) then
              !print*,'Get KPPh'
           endif
           CALL GetKPPh(KPPRIB,KPPHIND,hlim)
        else
           KPPh=0.0
        endif
        call exchange2d_mpi(KPPh,im,jm)
!/---------------------------------------------------------------------+
! Get KPP surface layer kh/km
!\---------------------------------------------------------------------+
        if (KPP_SBL) then
           if (diagnostic) then
              !print*,'Compute KH/KM'
           endif
           CALL KPPsurface(KPPHind,ustar2,bflux)
           if (diagnostic) then
              print*,'KM',minval(KMKPP),maxval(KMKPP)
              print*,'KH',minval(KHKPP),maxval(KHKPP)
           endif
        endif



        if (KPP_BBL) then
           call KPPbottom(ustar2)
        endif


        do indk=1,kb
           call exchange2d_mpi(kmkpp(:,:,indk),im,jm)
           call exchange2d_mpi(khkpp(:,:,indk),im,jm)
        enddo

        if (use_KPP_K) THEN
           do indj=1,jm
              do indi=1,im
                 do indk=1,kb
                 !if (ustar2(indi,indj).gt.2.9820e-05) then
                    km(indi,indj,indk)=min(max(1.0e-6,
     $                   kmkpp(indi,indj,indk)),1.)
                    kh(indi,indj,indk)=min(max(1.0e-6,
     $                   khkpp(indi,indj,indk)),1.)
                 !endif
                 enddo
              enddo
           enddo
        endif


! cosmetics: make boundr. values as interior (even if not used, printout
! may show strange values)
        do indk=1,kb
           do indi=1,im
              if(n_north.eq.-1) then
                 km(indi,jm,indk)=km(indi,jmm1,indk)
                 kh(indi,jm,indk)=kh(indi,jmm1,indk)
              end if
              if(n_south.eq.-1) then
                 km(indi,1,indk)=km(indi,2,indk)
                 kh(indi,1,indk)=kh(indi,2,indk)
              end if
           end do
           do indj=1,jm
              if(n_east.eq.-1) then
                 km(im,indj,indk)=km(imm1,indj,indk)
                 kh(im,indj,indk)=kh(imm1,indj,indk)
              end if
              if(n_west.eq.-1) then
                 km(1,indj,indk)=km(2,indj,indk)
                 kh(1,indj,indk)=kh(2,indj,indk)
              end if
           end do
        end do
        
        do indk=1,kb
           do indi=1,im
              do indj=1,jm
                 km(indi,indj,indk)=km(indi,indj,indk)*fsm(indi,indj)
                 kh(indi,indj,indk)=kh(indi,indj,indk)*fsm(indi,indj)
              end do
           end do
        end do

        if (diagnostic) then
           TOUTTEST=MOD(time,0.010416666666667)
           !if (TOUTTEST.lt.0.001) then
              print*,'time',time
              print*,'KPPh:',maxval(KPPh)
              print*,'T: ',maxval(t)
              print*,'hflux',minval(hflux),maxval(hflux)
              print*,'wtsurf',minval(wtsurf),maxval(wtsurf)
              print*,'swrad',minval(swrad),maxval(swrad)
              print*,'GAMT',minval(gammat),maxval(gammat)
              print*,'KM',minval(KMKPP),maxval(KMKPP)
              print*,'KH',minval(KHKPP),maxval(KHKPP)
           !endif
        endif
        
        RETURN


        
        end Subroutine profKPP
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
        SUBROUTINE getalpha(S,T,D,ALPHA_TE)
        real, intent(in) :: S, T, D
        real, intent(out) :: ALPHA_TE
        real :: Buoy1, Buoy2, Rho1, Rho2
        REAL, parameter :: DELTA=0.01
        
        call eqstate(S,T+0.5*DELTA,0.0,Rho1)
        call eqstate(S,T-0.5*DELTA,0.0,Rho2)
        Buoy1 = (Rho1-rhoref)/rhoref
        Buoy2 = (Rho2-rhoref)/rhoref
        
        ALPHA_TE = (Buoy1 - Buoy2) / DELTA
        return
        end Subroutine getalpha
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
        SUBROUTINE eqstate(SI,TI,DEPTH,potrho)
       !Calculate density
        !Needed for KPP
        real, intent (IN) :: TI, SI,DEPTH
        REAL, INTENT(OUT) :: potrho
        real cr, tr,sr,tr2,tr3,tr4,p,rhor
        
        tr=ti
        sr=si
        tr2=tr*tr
        tr3=tr2*tr
        tr4=tr3*tr
        
        p=grav*rhoref*(DEPTH)*1.e-5
        
        rhor=-0.157406e0+6.793952e-2*tr
     $       -9.095290e-3*tr2+1.001685e-4*tr3
     $       -1.120083e-6*tr4+6.536332e-9*tr4*tr
        
        rhor=rhor+(0.824493e0-4.0899e-3*tr
     $       +7.6438e-5*tr2-8.2467e-7*tr3
     $       +5.3875e-9*tr4)*sr
     $       +(-5.72466e-3+1.0227e-4*tr
     $       -1.6546e-6*tr2)*abs(sr)**1.5
     $       +4.8314e-4*sr*sr
        
        cr=1449.1e0+.0821e0*p+4.55e0*tr-.045e0*tr2
     $       +1.34e0*(sr-35.e0)
        rhor=rhor+1.e5*p/(cr*cr)*(1.e0-2.e0*p/(cr*cr))
        POTRHO=rhor+1000.
        
        RETURN
        
        end SUBROUTINE eqstate
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
        SUBROUTINE VSCALE(MOL,UST,D,WM,WS)
        REAL, intent(in):: MOL, UST,D
        REAL, intent(out) :: WM, WS
        REAL :: ZETA
        
        ZETA=D/(MOL+1e-20);
        
        if (ZETA.ge.0.) then
           WM=1./(1.+5.*ZETA);
           WS=WM;
        else
           if (ZETA.gt.ZETAm) then
              WM=1./((1.-16.*ZETA)**(-.25))
           else
              WM=1./((Am-Cm*ZETA)**(-.3333))
           endif
           if (ZETA.gt.ZETAs) then
              WS=1./((1.-16.*ZETA)**(-.5))
           else
              WS=1./((As-Cs*ZETA)**(-.3333))
           endif
        endif
        WM=kappa*UST*WM;
        WS=kappa*UST*WS;
        
        return
        end subroutine Vscale
      
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
      Subroutine Freq_Prof(SS,NN)
      !Calculate buoyancy and shear frequency profile
      ! Note that buoyancy and shear frequencies are defined on the interface
      real, intent(out), dimension(im,jm,kb) :: SS, NN
      integer :: indj, indi, indk
      real :: depth, rhou, rhol, drhoz, uup, vup, udn, vdn
      SS = 0.0
      NN = 1.e-8
      do indj=1,jmm1
         do indi=1,imm1
            if (fsm(indi,indj).gt.0.1) then
               SS(indi,indj,:)=0.
               SS(indi,indj,:)=0.
               NN(indi,indj,:)=1.e-4
               NN(indi,indj,:)=1.e-4
               do indk=2,kbm1
                  ! Depth of Mid-Point
                  DEPTH = 0.0 - zz(indk) * h(indi,indj)
                  call eqstate(S(indi,indj,indk-1),
     $                 T(indi,indj,indk-1),DEPTH,RHOU)
                  call eqstate(S(indi,indj,indk),
     $                 T(indi,indj,indk),
     $                 DEPTH,RHOL)
                  DRHOz = RHOU - RHOL
                  NN(indi,indj,indk) = ((-grav/(.5*RHOL+.5*RHOU)
     $                 *(DRHOz)/((dzz(indk-1))*h(indi,indj))))
                  Uup=.5*(u(indi,indj,indk-1)*dum(indi,indj)
     $                 +u(indi+1,indj,indk-1)*dum(indi+1,indj))
                  Vup=.5*(v(indi,indj,indk-1)*dvm(indi,indj)
     $                 +v(indi,indj+1,indk-1)*dvm(indi,indj+1))
                  Udn=.5*(u(indi,indj,indk)*dum(indi,indj)
     $                 +u(indi+1,indj,indk)*dum(indi+1,indj))
                  Vdn=.5*(v(indi,indj,indk)*dvm(indi,indj)
     $                 +v(indi,indj+1,indk)*dvm(indi,indj+1))
                  SS(indi,indj,indk) = ((Uup-Udn)**2 +(Vup-Vdn)**2)
     $                 /(dzz(indk-1)*h(indi,indj))**2
               enddo
            endif
         enddo
      enddo

      return
      end subroutine Freq_Prof
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
      Subroutine InteriorK(SS,NN)
      !Calculate interior viscosity/diffusivity
      real, intent(in), dimension(im,jm,kb) :: SS, NN
      real :: RIG(kb), nu_sx, nu_sxc, shear2
      integer :: indj, indi, indk

      RIG=0.0
      

      do indj=1,jmm1
         do indi=1,imm1
            if (fsm(indi,indj).gt.0.1) then
               do indk=1,kb
                  RIG(indk)=NN(indi,indj,indk)
     $                 /(SS(indi,indj,indk)+1.E-14)
               enddo
               do indk=kbm1,2,-1
                  if (indk.ge.2.and.indk.lt.kb) then
                     ! Gradient Richardson mixing (smoothing)
                     RIG(indk)=0.25*Rig(indk-1) + 0.50*Rig(indk) 
     $                    + 0.25*Rig(indk+1)
                     nu_sx = min(1.,max(0.,Rig(indk))/Ri0)
                     nu_sx = (1.0 - nu_sx*nu_sx)
                     nu_sx = nu_sx * nu_sx * nu_sx
                     shear2 = SS(indi,indj,indk)
                     nu_sx=nu_sx*shear2*shear2/
     $                    (shear2*shear2+16.0E-10)
                     nu_sxc = max(NN(indi,indj,indk),bvfcon)
                     nu_sxc = min(1.,(bvfcon-nu_sxc)/bvfcon)
                     nu_sxc = 1. - nu_sxc
                     nu_sxc = nu_sxc*nu_sxc*nu_sxc
                     kmkpp(indi,indj,indk)= nu_sxc*KC+nu_sx*KMRIg+KMWB
                     khkpp(indi,indj,indk)= nu_sxc*KC+nu_sx*KHRIg+KHWB
                  else
                     kmkpp(indi,indj,indk)=0.0
                     khkpp(indi,indj,indk)=0.0
                  endif
               enddo
            endif
         enddo
      enddo


      return
      end subroutine InteriorK
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
      Subroutine ComputeRIb(NN,UST2,BFLUX,KPPRIB)
      real, intent(in), dimension(im,jm,kb) :: NN
      real, intent(in), dimension(im,jm) :: UST2,BFLUX
      real, intent(out), dimension(im,jm,kb) :: KPPRIB
      real, dimension(im,jm) :: AVGDPTH
      integer :: indi, indj, indk
      logical :: first
      integer :: kstrt
      real :: ustop, vstop, DEPTH, RHOSURF, USURF, VSURF, DUz, DVz,RHOU, 
     $     RHOL, DRHOz, RItop, FLT, VT2, VTC, RIBOT, WM, WS

      AVGDPTH=0.0
      VTC = 1.6*sqrt(.2)/(sqrt(98.96*0.1)*KPPRic*.4*.4)
      do indj=1,jmm1
         do indi=1,imm1
            if (fsm(indi,indj).gt.0.1) then
               first=.true.
               kstrt=1
               ustop=0.0;
               vstop=0.0
               ! Determine thickness of top level
               DEPTH = dz(1) * h(indi,indj)
               ! compute density of top level
               call eqstate(S(indi,indj,1),T(indi,indj,1),
     $              DEPTH,rhosurf)
               rhosurf=rhosurf*DEPTH
               
               ! Compute u-component in top level
               ! If right point is not land average to get
               ! value at grid center

               if (kpplagrangian.eq.1) then
                  ustop = 0.5e0*(us(indi,indj,1)*dum(indi,indj)
     $                 +us(indi+1,indj,1)*dum(indi+1,indj))
                  vstop = 0.5e0*(vs(indi,indj,1)*dvm(indi,indj)
     $                 +vs(indi,indj+1,1)*dvm(indi,indj+1))
               endif
               
               usurf = (.5e0*(u(indi,indj,1)*dum(indi,indj)
     $              +u(indi+1,indj,1)*dum(indi+1,indj))+ustop)
     $              *DEPTH
               vsurf = (.5e0*(v(indi,indj,1)*dvm(indi,indj)
     $              +v(indi,indj+1,1)*dvm(indi,indj+1))+vstop)
     $              *DEPTH
               ! Determine how deep to average for surface layer referenced value
               AVGDPTH(indi,indj) =
     $              max(KPPh(indi,indj)*0.1,dz(1)*h(indi,indj))
               !AVGDPTH = max(AVGDPTH,2.5)
               do indk=2,kbm1
                 !Depth of bottom of cell to see if whole cell is
                 ! within SBL
                  DEPTH = 0.0 - z(indk+1) * h(indi,indj)
                  if (depth.lt.AVGDPTH(indi,indj)) then

                  ! Getting u-component/v-component
                     if (kpplagrangian.eq.1) then
                        ustop = 0.5e0*(
     $                       us(indi,indj,indk)*dum(indi,indj)
     $                       +us(indi+1,indj,indk)*dum(indi+1,indj))
                        vstop = 0.5e0*(
     $                       vs(indi,indj,indk)*dvm(indi,indj)
     $                       +vs(indi,indj+1,indk)*dvm(indi,indj+1))
                     endif
                     usurf = usurf+(.5e0*
     $                    (u(indi,indj,indk)*dum(indi,indj)
     $                    +u(indi+1,indj,indk)*dum(indi+1,indj))
     $                    +ustop)*dz(indk)*h(indi,indj)
                     vsurf = vsurf+(.5e0*
     $                    (v(indi,indj,indk)*dvm(indi,indj)
     $                    +v(indi,indj+1,indk)*dvm(indi,indj+1))
     $                    +vstop)*dz(indk)*h(indi,indj)
                  elseif (first) then
                     kstrt=indk
                     
                     if (kpplagrangian.eq.1) then
                        ustop = 0.5e0*(
     $                       us(indi,indj,indk)*dum(indi,indj)
     $                       +us(indi+1,indj,indk)*dum(indi+1,indj))
                        vstop = 0.5e0*(
     $                       vs(indi,indj,indk)*dvm(indi,indj)
     $                       +vs(indi,indj+1,indk)*dvm(indi,indj+1))                  \
                        
                     endif
                     
                     usurf = usurf+(.5e0*
     $                    (u(indi,indj,indk)*dum(indi,indj)
     $                    +u(indi+1,indj,indk)*dum(indi+1,indj))
     $                    +ustop)*(AVGDPTH(indi,indj)
     $                    +z(indk)*h(indi,indj))
                     vsurf = vsurf+(.5e0*
     $                    (v(indi,indj,indk)*dvm(indi,indj)
     $                      +v(indi,indj+1,indk)*dvm(indi,indj+1))
     $                    +vstop)*(AVGDPTH(indi,indj)
     $                    +z(indk)*h(indi,indj))
                     first=.false.
                     
                  endif
               enddo
               usurf = usurf/AVGDPTH(indi,indj)
               vsurf = vsurf/AVGDPTH(indi,indj)
               KPPRIB(indi,indj,:)=0.0
               do indk=kstrt,kbm1
                  if (kpplagrangian.eq.1) then
                     ustop = 0.5e0*(
     $                    us(indi,indj,indk)*dum(indi,indj)
     $                    +us(indi+1,indj,indk)*dum(indi+1,indj))
                     vstop = 0.5e0*(
     $                    vs(indi,indj,indk)*dvm(indi,indj)
     $                    +vs(indi,indj+1,indk)*dvm(indi,indj+1))
                  endif
                    !Depth on ZZ since u
                  DEPTH = 0.0 - zz(indk) * h(indi,indj) !is surf 0.0? et/el?
                  DUz = usurf - (.5e0*(
     $                 u(indi,indj,indk)*dum(indi,indj)
     $                 +u(indi+1,indj,indk)*dum(indi+1,indj))+
     $                 ustop)
                  DVz = vsurf - (.5e0*(
     $                 v(indi,indj,indk)*dvm(indi,indj)
     $                 +v(indi,indj+1,indk)*dvm(indi,indj+1))+
     $                 vstop)
                  
                  call eqstate(S(indi,indj,1),T(indi,indj,1),
     $                 DEPTH,rhosurf)
                  call eqstate(S(indi,indj,indk),
     $                 T(indi,indj,indk),DEPTH,RHOU)
                  DRHOz=rhosurf-(RHOU)
                  RItop = -grav / rhoref * DRHOz * DEPTH
                  FLT = min(1.0+2.3/(LA(indi,indj)**.5),50.0)
                  if (Bflux(indi,indj).ge.0.0) then
                     call VSCALE(HMO(indi,indj),
     $                    sqrt(ust2(indi,indj)),DEPTH,WM,WS)
                  else
                     call VSCALE(HMO(indi,indj),
     $                    sqrt(ust2(indi,indj)),
     $                    min(0.1*KPPH(indi,indj),DEPTH),WM,WS)
                  endif
                  VT2   = VTC * DEPTH * sqrt(abs(NN(indi,indj,indk)))
     $                 * WS*FLT
                  RIbot = DUz**2 + DVz**2 + VT2
                  KPPRIB (indi,indj,indk) = RItop / (RIbot +
     $                 SMALL)
               enddo
            endif
         enddo
      enddo
      return
      end subroutine ComputeRIb
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
      subroutine GetKPPh(KPPRIB,KPPHIND,HLIM)
      REAL, INTENT(IN),DIMENSION(IM,JM,KB) :: KPPRIB
      REAL, INTENT(IN),DIMENSION(IM,JM) :: HLIM
      INTEGER, INTENT(OUT),DIMENSION(IM,JM)::KPPHIND
      integer :: indj, indi, indk
      logical :: TRIG

      kpphind=1
      kpph=1.0

      do indj=1,jmm1
         do indi=1,imm1
            if (fsm(indi,indj).gt.0.1) then
               TRIG=.true.
               KPPh(indi,indj) = 1.0
               do indk=2,kbm1
                  if (KPPRIB(indi,indj,indk).gt.KPPRIc .AND.trig) then
                     KPPh(indi,indj) = 0.0 - zz(indk) * h(indi,indj)
                     KPPh(indi,indj) = KPPh(indi,indj)-
     $                    (KPPRIB(indi,indj,indk)-KPPRic)*dzz(indk)
     $                    *h(indi,indj)/(KPPRIB(indi,indj,indk)-
     $                    KPPRIB(indi,indj,indk-1))
                     KPPH(indi,indj)=min(hlim(indi,indj),
     $                    kpph(indi,indj))
                     trig=.false.
                  endif
               enddo
               if (trig) then
                  KPPh(indi,indj)=abs(Z(kb)*h(indi,indj))
               endif
               if (kpph(indi,indj).le.abs(h(indi,indj)*(Z(1)))) then
                  KPPH(indi,indj)=abs(h(indi,indj)*(Z(1)))
               endif
               KPPHind(indi,indj)=kb
               trig=.true.
               do indk=2,kb
                  if (trig.and.
     $                abs(Z(indk)*h(indi,indj)).gt.KPPh(indi,indj)) then
                     KPPHind(indi,indj)=indk
                     trig=.false.
                  endif
               enddo
            else
               KPPh(indi,indj)=0.0
            endif
            !if (kpph(indi,indj).gt.200.) then
            !   print*,'---------------'
            !   print*,'RIB',kpprib(indi,indj,:)
            !   print*,'---------------'
            !   print*,'u/v',u(indi,indj,:),v(indi,indj,:)
            !   print*,'---------------'
            !   print*,'t',t(indi,indj,:)
            !   print*,'---------------'
            !   stop
            !endif
         enddo
      enddo
      return
      end subroutine getKPPH
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
      subroutine KPPsurface(KPPHind,UST2,Bflux)

      REAL, intent(in),dimension(im,jm) :: UST2, Bflux
      INTEGER, intent(in),dimension(im,jm) :: KPPHind
      REAL :: WM, WS, DEPTH, SIG, FLT, FLT2
      INTEGER :: KSBL, indk, indi, indj
      REAL :: CFF, CFF_DN, CFF_UP, K_BL, dk_bl, gm1, dgm1ds, gdm1ds,
     $     gt1, dgt1ds, f1, a1, a2, a3, gm, gt

      GAMMAT=0.0
      GAMMAS=0.0
      GAMMAM=0.0

      
      do indi=1,imm1
         do indj=1,jmm1
            if (fsm(indi,indj).gt.0.1) then
               FLT = min(1.0+1./LA(indi,indj),2.25)
               if (SW_Matching) then
                                !Compute non-dimensional shapefunction as in GOTM
                  f1=5.*max(Bflux(indi,indj),0.)/(Ust2(indi,indj)**2
     $                 +small);
                  DEPTH=KPPh(indi,indj)
                  if (Bflux(indi,indj).ge.0.0) then
                     call VSCALE(HMO(indi,indj),
     $                    sqrt(ust2(indi,indj)),DEPTH,WM,WS)
                  else
                     call VSCALE(HMO(indi,indj),
     $                    sqrt(ust2(indi,indj)),
     $                    min(0.1*KPPH(indi,indj),DEPTH),WM,WS)
                  endif
                  ksbl=KPPhind(indi,indj)
                  cff=1./abs(DZ(ksbl-1)*h(indi,indj));
                  cff_dn=(abs(Z(ksbl)*h(indi,indj))
     $                 -KPPh(indi,indj))*cff
                  cff_up=(KPPh(indi,indj)-abs(Z(ksbl-1)*h(indi,indj)))
     $                   *cff
                  
                  K_bl   = cff_dn*KMKPP(indi,indj,ksbl)+
     $                 cff_up*KMKPP(indi,indj,ksbl-1);
                  dK_bl  = cff*(KMKPP(indi,indj,ksbl)
     $                 -KMKPP(indi,indj,ksbl-1));
                  Gm1    = K_bl/(abs(KPPh(indi,indj))*WM+small)
                  dGm1dS = -dK_bl/(WM+small) + K_bl*f1;
                  
                  K_bl   = cff_dn*KHKPP(indi,indj,ksbl)
     $                 +cff_up*KHKPP(indi,indj,ksbl-1);
                  dK_bl  = cff*(KHKPP(indi,indj,ksbl)
     $                 -KHKPP(indi,indj,ksbl-1));
                  Gt1    = K_bl/(abs(KPPh(indi,indj))*WS+small)
                  dGt1dS = -dK_bl/(WS+small) + K_bl*f1;
               endif
               do indk=1,kb
                  ! on z's (w levels)
                  DEPTH = 0.0 - z(indk) * h(indi,indj) !is surf 0.0? et/el?
                  if (DEPTH.lt.KPPH(indi,indj)) then
                     if (Bflux(indi,indj).ge.0.0) then
                        call VSCALE(HMO(indi,indj),
     $                       sqrt(ust2(indi,indj)),DEPTH,WM,WS)
                     else
                        call VSCALE(HMO(indi,indj),
     $                       sqrt(ust2(indi,indj)),
     $                       min(0.1*KPPH(indi,indj),DEPTH),WM,WS)
                     endif
                     SIG=min(1.,DEPTH/KPPH(indi,indj));
                     A1    = SIG-2.0;
                     A2    = 3.0-2.0*SIG;
                     A3    = SIG-1.0;
                     if (SW_MATCHING) THEN
                        Gm = A1+A2*Gm1!+A3*dGm1dS;
                        Gt = A1+A2*Gt1!+A3*dGt1dS;
                     else
                        GM=A1
                        GT=A1
                     endif
                     FLT2=1.+(FLT-1.)*min(1.,SIG*(1.-SIG)**2.0/0.1481)
                     kmkpp(indi,indj,indk)=DEPTH*WM*(1.+SIG*Gm)*FLT2
                     khkpp(indi,indj,indk)=DEPTH*WS*(1.+SIG*Gt)*FLT2
                     if (bflux(indi,indj).lt.0.0) then
                        GAMMAT(indi,indj,indk)=SIG*(1-SIG)**2
     $                       *SEEes*max(0.0,(WTSURF(indi,indj)-
     $                        swrad(indi,indj))+
     $                       BR(indi,indj)/rhoref/Cp)!subtract short wave
                        GAMMAS(indi,indj,indk)=SIG*(1+SIG*Gm)
     $                       *SEEes*max(0.0,WSSURF(indi,indj))
                     endif
                  endif
                  kmkpp(indi,indj,indk)=min(10.0,
     $                 kmkpp(indi,indj,indk))
                  khkpp(indi,indj,indk)=min(10.0,
     $                 khkpp(indi,indj,indk))
                  if (kmkpp(indi,indj,indk).lt.0.) then
                     print*,'Less than 0 check match'
                     print*,cff_dn/CFF,cff_up/CFF,ksbl,indk
                     print*,Z(ksbl)*h(indi,indj),Z(ksbl-1)*h(indi,indj)
     $                    ,KPPh(indi,indj)
                     print*,(1+SIG*Gm),dk_bl,k_bl
                     print*,A1,A2,GM1,A3,DGM1DS
                     print*,cff,(KHKPP(indi,indj,ksbl),
     $                 KHKPP(indi,indj,ksbl-1));
                     stop
                  endif
               enddo
            else
               do indk=1,kb
                  kmkpp(indi,indj,indk)=1.0e-5
                  khkpp(indi,indj,indk)=1.0e-5
               enddo
            endif
         enddo
      enddo

      return
      end subroutine KPPsurface
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
      subroutine KPPbottom(ust2)
      ! adding bottom boundary layer for use in KPP
      ! note this is a simple version of the KPP boundary layer scheme
      ! More complexity may/may not be worth it for this application...
      real, dimension(im,jm),intent(in) :: ust2
      real, dimension(im,jm) :: hek
      real, dimension(im,jm,kb) :: BOTRIB
      integer, dimension(im,jm) :: KPPbind
      REAL :: DEPTH, ubot,vbot,WM,WS
      REAL :: DUZ, DVZ, RHOU, DRHOZ, RITOP, RIBOT,RHOL
      REAL :: cff, cff_dn, cff_up, k_bl, dk_bl
      REAL :: GM1, dGM1DS,GT1,DGT1DS,A1,A2,A3,GM,GT,SIG,F1
      integer :: indi,indj,indk,kbbl
      logical :: first, TRIG

      KPPbind(:,:)=kb
      kppb(:,:)=0.01
      BOTRIB(:,:,:)=0.0
      hek(:,:) = 0.0
      ustarb2(:,:) = 0.0

      do indi=1,imm1
         do indj=1,jmm1
            ustarb2(indi,indj)= max(small,sqrt((.5e0*(wubot(indi,indj)
     $           * dum(indi,indj) + wubot(indi+1,indj)
     $           * dum(indi+1,indj)))**2 +
     $           (.5e0*(wvbot(indi,indj)*dvm(indi,indj)
     $           +wvbot(indi,indj+1)*dvm(indi,indj+1)))**2))
            hek(indi,indj) = 0.7 * sqrt(ustarb2(indi,indj))
     $                  / (small+cor(indi,indj))
         enddo
      enddo
!      print*,'hek1',minval(hek),maxval(hek)
      call exchange2d_mpi(ustarb2,im,jm)
      call exchange2d_mpi(hek,im,jm)

      do indj=1,jmm1
         do indi=1,imm1
            first=.true.
     
            DEPTH = 0.0 - zz(kbm1) * h(indi,indj)
     
            if (dum(indi+1,indj).gt.0.1) then
               ubot = .5e0*(u(indi,indj,kbm1)
     $              +u(indi+1,indj,kbm1))
            else
               ubot = .5e0*u(indi,indj,kbm1)
            endif
!     
            if (dvm(indi,indj+1).gt.0.1) then
               vbot = .5e0*(v(indi,indj,kbm1)
     $              +v(indi,indj+1,kbm1))
            else
               vbot = .5e0*v(indi,indj,kbm1)
            endif
            BOTRIB(indi,indj,:)=0.0
            do indk = kbm2,2,-1
               DEPTH = (zz(indk)-zz(kbm1)) * h(indi,indj)
               DUz = .5e0*(u(indi,indj,indk)
     $              +u(indi+1,indj,indk))-ubot
               DVz = .5e0*(v(indi,indj,indk)
     $              +v(indi,indj+1,indk))-vbot
               call eqstate(S(indi,indj,kbm1),
     $              T(indi,indj,kbm1),h(indi,indk),RHOL)
               call eqstate(S(indi,indj,indk),
     $              T(indi,indj,indk),h(indi,indk),RHOU)
               DRHOz=(RHOU)-RHOL
               RItop = max(0.0,-grav / RHOREF * DRHOz * DEPTH)
               RIbot= DUz**2 + DVz**2
              BOTRIB(indi,indj,indk) = RItop / (RIbot +
     $              1.e-8)
            enddo
         enddo
      enddo
     

      do indj=1,jmm1
         do indi=1,imm1
            TRIG=.true.
            KPPb(indi,indj) = 1.0
            if (botrib(indi,indj,kbm2).gt.KPPRIc) then
               KPPb(indi,indj) = 0.0+
     $                    (KPPRic)*dzz(kbm2)
     $                    *h(indi,indj)/(botRIB(indi,indj,kbm2)-0.0)
            else
               do indk=kbm2,1,-1
                  if (BOTRIB(indi,indj,indk).gt.KPPRIc .AND. trig) then
                     KPPb(indi,indj) = (zz(indk)-zz(kbm1))* h(indi,indj)
                     KPPb(indi,indj) = KPPb(indi,indj)-
     $                    (BOTRIB(indi,indj,indk)-KPPRic)*dzz(indk)
     $                    *h(indi,indj)/(botRIB(indi,indj,indk)-
     $                    botRIB(indi,indj,indk-1))
                     trig=.false.
                  elseif (zz(indk).gt.-0.5) then
                     kppb(indi,indj)=h(indi,indj)*.5
                     trig=.false.
                  endif
               enddo
            endif
            KPPb(indi,indj)=min(KPPb(indi,indj),h(indi,indj)*.5)
            KPPb(indi,indj)=min(hek(indi,indj),kppb(indi,indj))
            KPPb(indi,indj)=min(100.0,KPPb(indi,indj))
            KPPb(indi,indj)=max(0.1,KPPb(indi,indj))
            KPPbind(indi,indj)=kbm2
            trig=.true.
            do indk=kb,2,-1
               if (trig.and.
     $              abs((Z(indk)-Z(kb))*h(indi,indj))
     $              .gt.KPPb(indi,indj)) then
                  KPPbind(indi,indj)=indk
                  trig=.false.
               endif
            enddo
         enddo
      enddo
      Call exchange2d_mpi(KPPB,im,jm)
      call exchange2d_mpi(kppbind,im,jm)


      do indi=1,imm1
         do indj=1,jmm1
            WM = kappa*sqrt(ustarb2(indi,indj))
            WS = WM
            if (fsm(indi,indj).gt.0.1) then
               if (SW_Matching) then
                  if (kppbind(indi,indj).lt.kbm1) then
                     f1=0.0
                     kbbl=KPPbind(indi,indj)
                     cff=1./abs(DZ(kbbl+1)*h(indi,indj));
                     cff_dn=(abs(Z(kbbl+1)*h(indi,indj))
     $                    -(h(indi,indj)-KPPb(indi,indj)))*cff
                     cff_up=((h(indi,indj)-KPPb(indi,indj))
     $                    -abs(Z(kbbl)*h(indi,indj)))
     $                    *cff
                     K_bl   = cff_dn*KMKPP(indi,indj,kbbl+1)+
     $                    cff_up*KMKPP(indi,indj,kbbl);
                     dK_bl  = cff*(KMKPP(indi,indj,kbbl+1)
     $                    -KMKPP(indi,indj,kbbl));
                     Gm1    = K_bl/(abs(KPPb(indi,indj))*WM+small)
                     dGm1dS = -dK_bl/(WM+small) + K_bl*f1;
                     
                     K_bl   = cff_dn*KHKPP(indi,indj,kbbl+1)
     $                    +cff_up*KHKPP(indi,indj,kbbl);
                     dK_bl  = cff*(KHKPP(indi,indj,kbbl+1)
     $                    -KHKPP(indi,indj,kbbl));
                     Gt1    = K_bl/(abs(KPPb(indi,indj))*WS+small)
                     dGt1dS = -dK_bl/(WS+small) + K_bl*f1;
                  else
                     GT1=0.0
                     GM1=0.0
                  endif
               endif
               do indk=1,kbm1
                  ! on z's (w levels)
                  DEPTH = (z(indk) - z(kb)) * h(indi,indj) !is surf 0.0? et/el?
                  IF (DEPTH.lt.KPPB(indi,indj)) then
                     SIG=min(1.,DEPTH/KPPb(indi,indj));
                     A1    = SIG-2.0;
                     A2    = 3.0-2.0*SIG;
                     A3    = SIG-1.0;
                     if (SW_MATCHING) THEN
                        Gm = A1+A2*Gm1!+A3*dGm1dS; Dont match derivatives
                        Gt = A1+A2*Gt1!+A3*dGt1dS; 
                     else
                        GM=A1
                        GT=A1
                     endif
                     kmkpp(indi,indj,indk)=max(1.e-5,
     $                    min(10.0,DEPTH*WM*(1.+SIG*Gm)))
                     khkpp(indi,indj,indk)=max(1.e-5,
     $                    min(10.0,DEPTH*WS*(1.+SIG*Gt)))
                  endif
               enddo
            else
               do indk=1,kbm1
                  kmkpp(indi,indj,indk)=1.e-5
                  khkpp(indi,indj,indk)=1.e-5
               enddo
            endif
         enddo
      enddo
!      print*,'KPPb',minval(kppb),maxval(kppb)
!      print*,'hek',minval(hek),maxval(hek)
!      print*,'ustarb2',minval(ustarb2),maxval(ustarb2)
      end subroutine KPPbottom
!/----------------------------------------------------------------------
!\----------------------------------------------------------------------
      end module KPP
